﻿Imports System.ComponentModel.DataAnnotations
Imports System.ServiceModel.DomainServices.Client
Imports System.ServiceModel.DomainServices.Client.ApplicationServices
Imports CSSCustomerEdit.Web.Resources

Namespace CSSCustomerEdit.LoginUI

	''' <summary>
	''' This internal entity is used to ease the binding between the UI controls
	''' (DataForm and the label displaying a validation error) and the log on
	''' credentials entered by the user.
	''' </summary>
	Public Class LoginInfo
		Inherits Entity

		Private _userName As String
        Private _rememberMe As Boolean
        Private _currentLoginOperation As LoginOperation

		''' <summary>
		''' Gets and sets the user name.
		''' </summary>
		<Display(Name := "UserNameLabel", ResourceType := GetType(RegistrationDataResources)), Required>
		Public Property UserName() As String
			Get
				Return Me._userName
			End Get

			Set(ByVal value As String)
				If Me._userName <> value Then
					Me.ValidateProperty("UserName", value)
					Me._userName = value
					Me.RaisePropertyChanged("UserName")
				End If
			End Set
		End Property

		''' <summary>
		''' Gets or sets a function that returns the password.
		''' </summary>
        Friend Property PasswordAccessor() As Func(Of String)
        'Friend Property PasswordAccessor() As String

		''' <summary>
		''' Gets and sets the password.
		''' </summary>
		<Display(Name := "PasswordLabel", ResourceType := GetType(RegistrationDataResources)), Required>
		Public Property Password() As String
			Get
                If (Me.PasswordAccessor Is Nothing) Then
                    Return String.Empty
                Else
                    Return Me.PasswordAccessor().Invoke()
                End If
			End Get
			Set(ByVal value As String)
				Me.ValidateProperty("Password", value)

				' Do not store the password in a private field as it should
				' not be stored in memory in plain-text.  Instead, the supplied
				' PasswordAccessor serves as the backing store for the value.

				Me.RaisePropertyChanged("Password")
			End Set
		End Property

		''' <summary>
		''' Gets and sets the value indicating whether the user's authentication information
		''' should be recorded for future logins.
		''' </summary>
		<Display(Name := "RememberMeLabel", ResourceType := GetType(ApplicationStrings))>
		Public Property RememberMe() As Boolean
			Get
				Return Me._rememberMe
			End Get

			Set(ByVal value As Boolean)
				If Me._rememberMe <> value Then
					Me.ValidateProperty("RememberMe", value)
					Me._rememberMe = value
					Me.RaisePropertyChanged("RememberMe")
				End If
			End Set
		End Property

		''' <summary>
		''' Gets or sets the current login operation.
        ''' </summary>
        Private Sub Logout()
            Me.CurrentLoginOperationChanged()
        End Sub


        Friend Property CurrentLoginOperation() As LoginOperation
            Get
                Return Me._currentLoginOperation
            End Get
            Set(ByVal value As LoginOperation)
                If Me._currentLoginOperation IsNot value Then
                    If Me._currentLoginOperation IsNot Nothing Then
                        RemoveHandler Me._currentLoginOperation.Completed,
                            AddressOf CurrentOperationChangedHandler
                    End If

                    Me._currentLoginOperation = value

                    If Me._currentLoginOperation IsNot Nothing Then
                        AddHandler Me._currentLoginOperation.Completed,
                            AddressOf CurrentOperationChangedHandler
                    End If

                    Me.CurrentLoginOperationChanged()
                End If
            End Set
        End Property

        Private Sub CurrentOperationChangedHandler(ByVal s As Object, ByVal e As EventArgs)
            Me.CurrentLoginOperationChanged()
        End Sub

        ''' <summary>
        ''' Gets a value indicating whether the user is presently being logged in.
        ''' </summary>
        <Display(AutoGenerateField:=False)>
        Public ReadOnly Property IsLoggingIn() As Boolean
            Get
                Return Me.CurrentLoginOperation IsNot Nothing AndAlso Not Me.CurrentLoginOperation.IsComplete
            End Get
        End Property

        ''' <summary>
        ''' Gets a value indicating whether the user can presently log in.
        ''' </summary>
        <Display(AutoGenerateField:=False)>
        Public ReadOnly Property CanLogIn() As Boolean
            Get
                Return Not Me.IsLoggingIn
            End Get
        End Property

        ''' <summary>
        ''' Raises operation-related property change notifications
        ''' when the current login operation changes.
        ''' </summary>
        Private Sub CurrentLoginOperationChanged()
            Me.RaisePropertyChanged("IsLoggingIn")
            Me.RaisePropertyChanged("CanLogIn")
        End Sub

        ''' <summary>
        ''' Creates a new <see cref="LoginParameters"/>
        ''' instance using the data stored in this entity.
        ''' </summary>
        Public Function ToLoginParameters() As LoginParameters
            Return New LoginParameters(Me.UserName, Me.Password, Me.RememberMe, Nothing)
        End Function

        Private Sub CurrentOperationChanged()
            Throw New NotImplementedException
        End Sub

    End Class
End Namespace
