﻿Imports System.ComponentModel.DataAnnotations
Imports System.ServiceModel.DomainServices.Client
Imports System.ServiceModel.DomainServices.Client.ApplicationServices
Imports CSSCustomerEdit.Web.Resources

Namespace CSSCustomerEdit.Web

	''' <summary>
	''' Extensions to provide client side custom validation and data binding to <see cref="RegistrationData"/>.
	''' </summary>
	Partial Public Class RegistrationData
        Inherits Entity
		Private _currentOperation As OperationBase

		''' <summary>
		''' Gets or sets a function that returns the password.
		''' </summary>
        Friend Property PasswordAccessor() As Func(Of String)


		''' <summary>
		''' Gets and sets the password.
		''' </summary>
        <Required(ErrorMessageResourceName:="ValidationErrorRequiredField",
            ErrorMessageResourceType:=GetType(ValidationErrorResources)),
        Display(Order:=3, Name:="PasswordLabel", Description:="PasswordDescription",
            ResourceType:=GetType(RegistrationDataResources)),
        RegularExpression("^.*[^a-zA-Z0-9].*$", ErrorMessageResourceName:="ValidationErrorBadPasswordStrength",
            ErrorMessageResourceType:=GetType(ValidationErrorResources)),
        StringLength(50, MinimumLength:=7, ErrorMessageResourceName:="ValidationErrorBadPasswordLength",
            ErrorMessageResourceType:=GetType(ValidationErrorResources))>
        Public Property Password() As String
            Get
                If (Me.PasswordAccessor Is Nothing) Then

                    Return String.Empty
                Else
                    Return Me.PasswordAccessor().Invoke()
                End If

            End Get

            Set(ByVal value As String)
                Me.ValidateProperty("Password", value)
                Me.CheckPasswordConfirmation()

                ' Do not store the password in a private field as it should
                ' not be stored in memory in plain-text.  Instead, the supplied
                ' PasswordAccessor serves as the backing store for the value.

                Me.RaisePropertyChanged("Password")
            End Set
        End Property

		''' <summary>
		''' Gets or sets a function that returns the password confirmation.
		''' </summary>
        Friend Property PasswordConfirmationAccessor() As Func(Of String)

		''' <summary>
		''' Gets and sets the password confirmation string.
		''' </summary>
        <Required(ErrorMessageResourceName:="ValidationErrorRequiredField",
            ErrorMessageResourceType:=GetType(ValidationErrorResources)),
        Display(Order:=4, Name:="PasswordConfirmationLabel", ResourceType:=GetType(RegistrationDataResources))>
        Public Property PasswordConfirmation() As String
            Get
                If (Me.PasswordConfirmationAccessor.Equals(Nothing)) Then
                    Return String.Empty
                Else
                    Return Me.PasswordConfirmationAccessor().Invoke()
                End If
            End Get

            Set(ByVal value As String)
                Me.ValidateProperty("PasswordConfirmation", value)
                Me.CheckPasswordConfirmation()

                ' Do not store the password in a private field as it should
                ' not be stored in memory in plain-text.  Instead, the supplied
                ' PasswordAccessor serves as the backing store for the value.

                Me.RaisePropertyChanged("PasswordConfirmation")
            End Set
        End Property

		''' <summary>
		''' Gets or sets the current registration or login operation.
		''' </summary>
		Friend Property CurrentOperation() As OperationBase
			Get
				Return Me._currentOperation
			End Get
			Set(ByVal value As OperationBase)
                If Me._currentOperation IsNot value Then
                    If Me._currentOperation IsNot Nothing Then
                        RemoveHandler Me._currentOperation.Completed, AddressOf CurrentOperationChangedHandler
                    End If

                    Me._currentOperation = value

                    If Me._currentOperation IsNot Nothing Then
                        AddHandler Me._currentOperation.Completed, AddressOf CurrentOperationChangedHandler
                    End If

                    Me.CurrentOperationChanged()
                End If
			End Set
		End Property

        Private Sub CurrentOperationChangedHandler(ByVal s As Object, ByVal e As EventArgs)
            Me.CurrentOperationChanged()
        End Sub

        ''' <summary>
        ''' Gets a value indicating whether the user is presently being registered or logged in.
        ''' </summary>
        <Display(AutoGenerateField:=False)>
        Public ReadOnly Property IsRegistering() As Boolean
            Get
                Return Me.CurrentOperation IsNot Nothing AndAlso Not Me.CurrentOperation.IsComplete
            End Get
        End Property

        ''' <summary>
        ''' Helper method for when the current operation changes.
        ''' Used to raise appropriate property change notifications.
        ''' </summary>
        Private Sub CurrentOperationChanged()
            Me.RaisePropertyChanged("IsRegistering")
        End Sub

        ''' <summary>
        ''' Checks to ensure the password and confirmation match.  If they don't match,
        ''' then a validation error is added.
        ''' </summary>
        Private Sub CheckPasswordConfirmation()
            ' If either of the passwords has not yet been entered, then don't test for equality between
            ' the fields.  The Required attribute will ensure a value has been entered for both fields.
            If String.IsNullOrWhiteSpace(Me.Password) OrElse
                String.IsNullOrWhiteSpace(Me.PasswordConfirmation) Then
                Return
            End If

            ' If the values are different, then add a validation error with both members specified
            If Me.Password <> Me.PasswordConfirmation Then
                Me.ValidationErrors.Add(
                    New ValidationResult(ValidationErrorResources.ValidationErrorPasswordConfirmationMismatch,
                                         New String() {"PasswordConfirmation", "Password"}))
            End If
        End Sub

        ''' <summary>
        ''' Perform logic after the UserName value has been entered
        ''' </summary>
        ''' <param name="userName">The user name value that was entered.</param>
        ''' <remarks>
        ''' Allow the form to indicate when the value has been completely entered, because
        ''' using the OnUserNameChanged method can lead to a premature call before the user
        ''' has finished entering the value in the form.
        ''' </remarks>
        Friend Sub UserNameEntered(ByVal userName As String)
            ' Auto-Fill FriendlyName to match UserName for new entities when there is not a friendly name specified
            If (Me.EntityState = EntityState.New OrElse
                Me.EntityState = EntityState.Detached) AndAlso
                String.IsNullOrWhiteSpace(Me.FriendlyName) Then
                Me.FriendlyName = userName
            End If
        End Sub

        ''' <summary>
        ''' Creates a new <see cref="LoginParameters"/>
        ''' initialized with this entity's data (IsPersistent will default to false)
        ''' </summary>
        Public Function ToLoginParameters() As LoginParameters
            Return New LoginParameters(Me.UserName, Me.Password, False, Nothing)
        End Function
    End Class
End Namespace
